/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_FSM_INTERNAL_FSM_H
#define LOUS_SRC_FSM_INTERNAL_FSM_H

#ident "@(#)fsm.h 1.7      04/02/21 Chris G. Davis"

/** \file src/fsm/internal/fsm.h
 *  \brief Finite state machine internal definitions
 *  \author Chris G. Davis
 *
*/

struct LOUS_FSM {
  /**
     list of fsm_state's
  */
  hash_t* states;
  fsm_state_t* initial;
  s32_t num_states;

  /**
     list of fsm_trigger's
  */
  hash_t* triggers;
  s32_t num_triggers;

  /**
   * current users of this fsm
   */
  hash_t* users;
  s32_t num_users;

  bool_t lock;

  void (*callback)(fsm_t*, char* tag, char* state, char* old);
};

struct LOUS_FSM_TRIGGER {
  iterator_entry_t iterate;
  char* tag;

  s32_t initial;
};

struct LOUS_FSM_STATE {
  iterator_entry_t iterate;
  char* tag;
  s32_t flags;
  s32_t val;

  /**
     list of fsm_events
  */
  hash_t* events;
  s32_t num_events;

  void (*callback)(fsm_t*, char* tag);
};

struct LOUS_FSM_EVENT_PARAM {
  iterator_entry_t iterate;
  char* trigger_tag;

  s32_t val;
};
typedef struct LOUS_FSM_EVENT_PARAM fsm_event_param_t;

struct LOUS_FSM_EVENT {
  iterator_entry_t iterate;
  char* tag;

  char* target_tag;
  fsm_state_t* target;

  hash_t* params;
  s32_t num_params;
};

struct LOUS_FSM_USER {
  iterator_entry_t iterate;
  char* tag;

  fsm_t* fsm;

  struct timeval last_state_change;

  hash_t* trigger_state;
  bool_t been_run;

  fsm_state_t* curr_state;
  fsm_state_t* prev_state;
};

/*
 * event.c
 */
fsm_event_param_t* fsm_trigger_build_param( fsm_trigger_t* trigger);
s32_t fsm_event_fixup(fsm_t* fsm, fsm_state_t* state, fsm_event_t* event);
s32_t fsm_event_matches(fsm_event_t* event, hash_t* trigger_state);

/*
 * state.c
 */
s32_t fsm_state_fixup(fsm_t* fsm, fsm_state_t* state);

/*
 * user.c
 */
void fsm_user_run(fsm_t* fsm, fsm_user_t* user);

#endif /* LOUS_SRC_FSM_INTERNAL_FSM_H */
