/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_ALLOC_H
#define LOUS_SRC_INCLUDE_ALLOC_H

#ident "@(#)alloc.h 1.4      04/02/21 Chris G. Davis"

/** \file src/include/alloc.h
 *  \brief Memory allocation definitions
 *  \author Chris G. Davis
 *
*/

#ifdef LOUS_DEBUG

#define Lmalloc(target, size, retval, ...) \
if( ( target = malloc( size )) == NULL ) {\
logging_log(lous_loggroup, -1, 0, "[%s %d %s]", __FILE__, __LINE__, __FUNCTION__); \
logging_log(lous_loggroup, -1, 0, " (%d) %s", errno, strerror(errno)); \
logging_log(lous_loggroup, -1, 0, "Malloc failed");\
 __VA_ARGS__ \
 return retval; \
} else { \
  memset( target, 0, size); \
} while(0) {}

#define Lstrdup(target, source, retval, ...) \
if( ( target = strdup( source )) == NULL ) {\
logging_log(lous_loggroup, -1, 0, "[%s %d %s]", __FILE__, __LINE__, __FUNCTION__); \
logging_log(lous_loggroup, -1, 0, " (%d) %s", errno, strerror(errno)); \
logging_log(lous_loggroup, -1, 0, "strdup failed");\
 __VA_ARGS__ \
 return retval; \
} while(0) {}

#else

#define Lmalloc(target, size, retval, ...) \
if( ( target = malloc( size )) == NULL ) {\
 __VA_ARGS__ \
 return retval; \
} else { \
  memset( target, 0, size ); \
} while(0) {}

#define Lstrdup(target, source, retval, ...) \
if( ( target = strdup( source )) == NULL ) {\
 __VA_ARGS__ \
 return retval; \
} while(0) {}

#endif

#define LOUS_MEMALLOC_SHOW_ALLOC 0x1
#define LOUS_MEMALLOC_SHOW_FREE 0x2

#ifdef LOUS_DEBUG_MEMALLOC

#define lalloc(size) lous_malloc(&__lous_counter, size, \
__FILE__, __FUNCTION__, __LINE__);
#define lfree(ptr) lous_free(&__lous_counter, ptr,\
__FILE__, __FUNCTION__, __LINE__);
#define lcalloc(num, size) lous_calloc(&__lous_counter, num, size,\
__FILE__, __FUNCTION__, __LINE__)l
#define lrealloc(ptr, size) lous_realloc(&__lous_counter, ptr, size, \
__FILE__, __FUNCTION__, __LINE__);

void* lous_calloc(s32_t* count, s32_t nmemb, s32_t size,
		  char* fl, char*fn, s32_t ln);
void* lous_malloc(s32_t* count, s32_t size,
		 char* fl, char*fn, s32_t ln);
void lous_free(s32_t* count, void *ptr,
		  char* fl, char*fn, s32_t ln);

void* lous_realloc(s32_t* count, void *ptr, s32_t size,
		   char* fl, char*fn, s32_t ln);


void lous_memalloc_initialize(FILE* out, s32_t flags);

void lous_memalloc_state();


#define LMEM() s32_t __lous_counter = 0
#define LEND() if( __lous_counter != 0 ) \
fprintf(stderr, "Alloc imbalance %s in %s\n", __FUNCTION__, __FILE__)

#define LLOSE(size) __lous_counter -= size
#define LFIND(size) __lous_counter += size
#define LINIT(out, flags) lous_memalloc_initialize(out, flags)
#define LFDISPLAY() lous_memalloc_func_state(&__lous_counter)
#define LDISPLAY() lous_memalloc_state()

#else

#define lalloc(size) malloc(size)
#define lfree(ptr) free(ptr)
#define lcalloc(num, size) calloc(num, size)
#define lrealloc(ptr, size) realloc(ptr, size)

#define LMEM(...)
#define LEND(...)
#define LLOSE(...)
#define LFIND(...)
#define LINIT(...)
#define LFDISPLAY(...)
#define LDISPLAY()

#endif /* LOUS_DEBUG_MEMALLOC */

#endif /* LOUS_SRC_INCLUDE_ALLOC_H */
