/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_FSM_H
#define LOUS_SRC_INCLUDE_FSM_H

#ident "@(#)fsm.h 1.11      04/02/21 Chris G. Davis"

/** \file src/include/fsm.h
 *  \brief Finite state machine definition
 *  \author Chris G. Davis
 *
*/

/*
 * Required LOUS functionality
 */
#include "lous/hash.h"
#include "lous/splay.h"

struct LOUS_FSM;
typedef struct LOUS_FSM fsm_t;

struct LOUS_FSM_TRIGGER;
typedef struct LOUS_FSM_TRIGGER fsm_trigger_t;

struct LOUS_FSM_STATE;
typedef struct LOUS_FSM_STATE fsm_state_t;

struct LOUS_FSM_EVENT;
typedef struct LOUS_FSM_EVENT fsm_event_t;

struct LOUS_FSM_USER;
typedef struct LOUS_FSM_USER fsm_user_t;


/* FSM */
#define FSM_DISPLAY_ADDR               0x0001

fsm_t* fsm_alloc(fsm_state_t* initial, void (*callback)(fsm_t*, char*, char*, char*));
void fsm_destroy(fsm_t* fsm);
s32_t fsm_lock(fsm_t* fsm);
void fsm_run(fsm_t* fsm);
void fsm_display_fsm(fsm_t* fsm, s32_t flags);
void fsm_set_trigger(fsm_t* fsm, char* user_tag, char* tag, s32_t val);
s32_t fsm_get_trigger(fsm_t* fsm, char* user_tag, char* tag);
void fsm_inc_trigger(fsm_t* fsm, char* user_tag, char* tag, s32_t val);
void fsm_force_state(fsm_t* fsm, char* user_tag, char* tag);
void fsm_display_tag(fsm_t* fsm, char*);

/* FSM STATE */
#define FSM_STATE_TIMEOUT              0x0001

fsm_state_t* fsm_state_alloc(char* tag, void (*callback)(fsm_t*, char*), s32_t flags, s32_t val);
//void fsm_state_destroy(fsm_state_t* state);
void fsm_state_add(fsm_t* fsm, fsm_state_t* state);
//void fsm_state_remove(fsm_t* fsm, char* tag);
void fsm_state_add_event(fsm_state_t* state, fsm_event_t* event);
//void fsm_state_remove_event(fsm_state_t* state, char* tag);
fsm_state_t* fsm_state_find(fsm_t* fsm, char* tag);
void fsm_state_display(fsm_state_t* state, s32_t flags);

/* FSM TRIGGER */
fsm_trigger_t* fsm_trigger_alloc(char* tag, s32_t initial);
//void fsm_trigger_destroy(fsm_trigger_t* trigger);
fsm_trigger_t* fsm_trigger_copy(fsm_trigger_t* trigger);
void fsm_trigger_add(fsm_t* fsm, fsm_trigger_t* trigger);
//void fsm_trigger_remove(fsm_t* fsm, char* tag);

/* FSM EVENT */
fsm_event_t* fsm_event_alloc(char* tag, char* nextstate);
//void fsm_event_destroy(fsm_event_t* event);
void fsm_event_add_param(fsm_event_t* event, char* trigger, s32_t val);
void fsm_event_display(fsm_event_t* event, s32_t flags);

/* FSM USER */
fsm_user_t* fsm_user_alloc(fsm_t* fsm, char* tag);
void fsm_user_destroy(fsm_user_t* user);
void fsm_user_set_trigger(fsm_user_t* user, char* trigger, s32_t val);
s32_t fsm_user_get_trigger(fsm_user_t* user, char* trigger);
void fsm_user_inc_trigger(fsm_user_t* user, char* trigger, s32_t val);
void fsm_user_display(fsm_user_t* user);
void fsm_user_force_state(fsm_user_t* fsm, char* state);

#endif /* LOUS_SRC_INCLUDE_FSM_H */
