/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_IPROUTE_H
#define LOUS_SRC_INCLUDE_IPROUTE_H

#ident "@(#)iproute.h 1.3      04/02/21 Chris G. Davis"

/** \file src/include/iproute.h
 *  \brief Networking interface layer
 *  \author Chris G. Davis
 *
 * \todo change to netint
 * \todo rework and recode
 */

char* findMinNetmask(char* ip, char* gip);

/*
 * headers and stuff from iproute2
 */
struct rtnl_handle
{
  int                     fd;
  struct sockaddr_nl      local;
  __u32                   seq;
  __u32                   dump;
};

int rtnl_open(struct rtnl_handle *rth, unsigned subscriptions);
void rtnl_close(struct rtnl_handle *rth);
int rtnl_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n, struct nlmsghdr *answer);
int rtnl_send(struct rtnl_handle *rth, char *buf, int len);
int rtnl_addattr32(struct nlmsghdr *n, int maxlen, int type, __u32 data);
int rtnl_addattr_l(struct nlmsghdr *n, int maxlen, int type, void *data, int alen);
int rtnl_wilddump_request(struct rtnl_handle *rth, int fam, int type);
int rtnl_dump_filter(struct rtnl_handle *rth,
                            int (*filter)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
		     void *arg1);
int rtnl_parse_rtattr(struct rtattr *tb[], int max, struct rtattr *rta, int len);

int ll_init_map(struct rtnl_handle *rth);
int ll_name_to_index(char *name);
int ll_free_map();
int ll_remember_index(struct sockaddr_nl *who, struct nlmsghdr *n, void *arg);

/*
 * iproute2 replacement functions
 */
int iproute_tunnel(int cmd, int encaps, char* tname, struct in_addr* local, struct in_addr* remote);
int iproute_tunnel_add(int encaps, char* tname, struct in_addr* local, struct in_addr* remote);
int iproute_tunnel_del(char* tname);
int iproute_addr_cmd(int cmd, char* ifname, struct in_addr* addr);
int iproute_addr_add(char* ifname, struct in_addr* addr);
int iproute_addr_del(char* ifname, struct in_addr* addr);
int iproute_link_set(char* ifname, int flag, int value);
int iproute_link_set_up(char* ifname);
int iproute_link_set_down(char* ifname);
int iproute_route(int cmd, int flags, int type, char* dev, struct in_addr* dest, int cidr, int table);
int iproute_route_add(char* dev, struct in_addr* dest, int table);
int iproute_route_del(char* dev, struct in_addr* dest, int table);

#endif /* LOUS_SRC_INCLUDE_IPROUTE_H */
