/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_LINKEDLIST_H
#define LOUS_SRC_INCLUDE_LINKEDLIST_H

#ident "@(#)linkedlist.h 1.8      04/02/21 Chris G. Davis"

/** \file src/include/linkedlist.h
 *  \brief linked list definitions
 *
 */

/*
 * Required LOUS functionality
 */
#include "lous/lous.h"

struct LOUS_LinkedListNode;
typedef struct LOUS_LinkedListNode linkedlist_node_t; /**< linked list node structure */

struct LOUS_LinkedList;
typedef struct LOUS_LinkedList linkedlist_t; /**< linked list structure */

/** Allocate a linked list
 * 
 * @param flags
 * @param free_func
 *
 * @return linked list structure
 */
linkedlist_t* ll_alloc(s32_t flags, void (*free_func)(void*));

/** Destroy a linked list
 * 
 * @param list
 */
s32_t ll_destroy(linkedlist_t* list);

/** Insert data item to the beginning of the list
 * 
 * @param list
 * @param data
 */
s32_t ll_insert_begin(linkedlist_t* list, void* data);

/** Insert data item to the end of the list
 * 
 * @param list
 * @param data
 */
s32_t ll_insert_end(linkedlist_t* list, void* data);

/** Insert data item at the current location in the list
 * 
 * @param list
 * @param data
 */
s32_t ll_insert_here(linkedlist_t* list, void* data);

/** Remove the next data item from the beginning of the list
 * 
 * @param list
 */
void* ll_remove_begin(linkedlist_t* list);

/** Remove the next data item from the end of the list
 * 
 * @param list
 */
void* ll_remove_end(linkedlist_t* list);

/** Remove the current data item
 * 
 * @param list
 */
void* ll_remove_here(linkedlist_t* list);

/** Return the first data item at the beginning of the list
 * 
 * @param list
 */
void* ll_getfirst(linkedlist_t* list);

/** Return the next data item
 * 
 * @param list
 */
void* ll_getnext(linkedlist_t* list);

/** Clear the current position in the list
 * 
 * @param list
 */
void ll_clearnext(linkedlist_t* list);

#endif /* LOUS_SRC_INCLUDE_LINKEDLIST_H */
