/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_MD5_H
#define LOUS_SRC_INCLUDE_MD5_H

#ident "@(#)md5.h 1.4      04/02/21 Chris G. Davis"

/** \file src/include/md5.h
 *  \brief Math md5 definitions
 *
 */

#define HIGHFIRST

#ifdef __i386__
#undef HIGHFIRST
#endif

struct MD5Context {
        u32_t buf[4];
        u32_t bits[2];
        u8_t in[64];
};

typedef struct MD5Context MD5_CTX; /**< md5 context structure */

/** Initialize the md5 structures
 * 
 */
void MD5Init();

/** Update the md5 structures
 * 
 */
void MD5Update();

/** Finalize the md5 structures
 * 
 */
void MD5Final();

/** Run a transformation on the md5 structures
 * 
 */
void MD5Transform();


/** md5 the data
 * 
 * @param key
 * @param keylen
 * @param data
 * @param datalen
 * @param output
 *
 * return 16 bytes of md5 data on \a output
 */
void md5(char* key, s32_t keylen, 
              char* data, s32_t datalen, char* output);

/** 
 * 
 * @param key
 * @param keylen
 * @param data
 * @param datalen
 * @param output
 *
 * return 16 bytes of md5 data on \a output
 */
void hmac_md5(char* key, s32_t keylen, 
              char* data, s32_t datalen, char* output);

#endif /* LOUS_SRC_INCLUDE_MD5_H */
