/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_PREGEX_H
#define LOUS_SRC_INCLUDE_PREGEX_H

#ident "@(#)pregex.h 1.1      04/02/21 Chris G. Davis"

/** \file src/include/pregex.h
 *  \brief Partial regular expression definitions
 *  \author Chris G. Davis
 *
*/

/*
 * Required LOUS functionality
 */
#include "lous/linkedlist.h"

struct Partial_Regex;
typedef struct Partial_Regex pregex_t; /**< partial regex structure */

enum RegexMatch { 
  matches = 0, /**< the string matches the regex */
  nomatch = -1, /**< the string does not match the regex */
  partialmatch = 1, /**< the string matches the regex as far as it goes */
  errmatch = -2, /**< some general error */
  magicmatch = -3, /**< bad magic in rx */
  parammatch = -4, /**< bad param */
 };
typedef enum RegexMatch regmatch_t;

#define PREGEX_ICASE       0x0001 /**< case-insensitive regex */

/** Allocate and compile a partial regex
 * 
 * @param string the regex string
 * @param flags
 *
 * @return the partial regex structure
 */
pregex_t* pregex_alloc(char* string, s32_t flags);

/** Destroy a partial regex structure
 *
 * @param rx 
 */
void pregex_destroy(pregex_t* rx);

/** Check the string agains the partial regex
 *
 * @param string
 * @param rx
 * @param flags
 *
 * @return the result of the pregex comparison
 */
regmatch_t pregex_check(char* string, pregex_t* rx, s32_t flags);

#endif /* LOUS_SRC_INCLUDE_PREGEX_H */
