/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_QUEUE_H
#define LOUS_SRC_INCLUDE_QUEUE_H

#ident "@(#)queue.h 1.4      04/02/21 Chris G. Davis"

/** \file src/include/queue.h
 *  \brief Queue definitions
 *
 */

/*
 * Required LOUS functionality
 */
#include "lous/linkedlist.h"

/*
 * Linked list types
 */
#define QUEUE_LIFO          0x10 /**< Last-In, First-Out Queue */
#define QUEUE_FIFO          0x20 /**< First-In, First-Out Queue */

/**  Allocate a queue
 * 
 * @param flags
 * @param free_func
 *
 * @return the queue structure
 */
linkedlist_t* queue_alloc(s32_t flags, void (*free_func)(void*));

/** Destroy the queue 
 * 
 * @param list
 */
s32_t queue_destroy(linkedlist_t* list);

/** Push data onto the queue
 * 
 * @param list
 * @param data
 */
s32_t queue_push(linkedlist_t* list, void* data);

/** Pop data off the queue
 * 
 * @param list
 *
 * @return the next piece of data on the stack
 */
void* queue_pop(linkedlist_t* list);

#endif /* LOUS_SRC_INCLUDE_QUEUE_H */
