/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_LOGGING_INTERNAL_LOGGING_H
#define LOUS_SRC_LOGGING_INTERNAL_LOGGING_H

#ident "@(#)logging.h 1.4      04/02/23 Chris G. Davis"

/** \file src/logging/internal/logging.h
 *  \brief General logging and debugging code
 *  \author Chris G. Davis
 *
 */

/** \brief The logging group structure
 *
 * The group structure holds the list of registered logging
 * classes and levels.
 *
 */
struct LOUS_Logging_Group {
  s32_t num_classes; /**< number of registered classes in this group */
  char* classes[LOGGING_MAX_GROUPS]; /**< class names for each registered class */
  s32_t class_users[LOGGING_MAX_GROUPS]; /**< number of users on each class */

  s32_t num_levels; /**< maximum number of logging levels */
  char* levels[LOGGING_MAX_LEVELS]; /**< level names fo each allowable level */
  s32_t level_users[LOGGING_MAX_LEVELS]; /**< number of users on each level */

  s32_t class_level_users[LOGGING_MAX_GROUPS][LOGGING_MAX_LEVELS];

  s32_t num_users; /**< number of users currently attached to this group */
  loguser_t* users; /**< users on this group */

  struct timeval currtime;
  bool_t fasttime;
};

/** \brief The logging user structure
 *
 * Each logging user is associated with one logging group, and
 * makes use of the classes and levels from that group
 */
struct LOUS_Logging_User {
  loggroup_t* group; /**< logging group currently associated with this user */

  s32_t classmask; /**< current class mask, with 1 bit for each class */

  s32_t class_level[LOGGING_MAX_GROUPS]; /**< current level of each class */
  logcallback_t class_callback[LOGGING_MAX_GROUPS]; /**< callback for each class */

  logcallback_t user_callback; /**< user callback */

  bool_t log_stdout; /**< log to stdout */
  bool_t log_stderr; /**< log to stderr */
  bool_t log_flush; /**< flush logging after each message */
  bool_t log_notime; /**< don't put time in the log message */
  bool_t log_nodate; /**< don't put date in the log message */

  void* logout; /**< log output data */
  s32_t flags; /**< flags for log user */
  loguser_t* next; /**< the next user in the group */
};

#endif /* */
