/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)uniqueid.c 1.5      04/02/21 Chris G. Davis"

/** \file src/math/uniqueid.c
 *  \brief Unique ID code
 *  \author Chris G. Davis 
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lous/uniqueid.h"
#include "internal/uniqueid.h"

unique_set_t* unique_set_alloc(s32_t min, s32_t max) {
  unique_set_t* retval;

  errno = 0;

  Lmalloc(retval, sizeof( unique_set_t ), NULL);

  retval->set = hash_alloc(128, 4, 
			   HASH_MULTIPLEX | HASH_SPLAY,
			   NULL, NULL, NULL);
  if( retval->set == NULL ) {
    free( retval );
    retval = NULL;
  } else {
    retval->min = min;
    retval->max = max;
  }

  return retval;
}

void unique_set_destroy(unique_set_t* set) {

  if( set != NULL ) {
    errno = 0;

    hash_destroy( set->set );
    free(set);

  } else {
    errno = ENOENT;
  }

}

s32_t unique_set_get(unique_set_t* set) {
  s32_t retval = -1;
  s32_t testval;
  s32_t count = 0;
  void* ret;

  errno = ENOENT;

  if( set != NULL ) {
    count = set->max - set->min + 1;
    
    errno = ENOENT;
    while( count-- ) {
      
      testval = random();
      
      testval = testval % ( set->max - set->min + 1 );
      
      testval += set->min;
      
      if( (ret = hash_lookup( set->set, &testval )) == NULL ) {
	retval = testval;
	hash_insert( set->set, &testval, "" );
	break;
      }
    }
  }

  return retval;
}

s32_t unique_set_put(unique_set_t* set, s32_t val) {
  s32_t retval = -1;

  errno = 0;

  if( set == NULL ) {
    errno = ENOENT;
  }

  retval = (hash_remove(set->set, &val ) == NULL );

  return retval;
}
