/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)netsocket.c 1.4      04/02/21 Chris G. Davis"

/** \file src/networking/netsocket.c
 *  \brief Socket code
 *  \author Chris G. Davis 
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/un.h>
#include <fcntl.h>
#include <net/if.h>

#include "lous/lous.h"
#include "lous/logging.h"
#include "lous/netsocket.h"

s32_t netsocket_alloc(s32_t flags, char* addr, s32_t port) {
  s32_t sckt;
  struct sockaddr_in saddr = {0};
  
  LDFunc();
  LDParam(flags, != flags, "%08x", -1);
  LDParam(addr, == NULL, "%s", -1);
  LDParam(port, != port, "%d", -1);

  if( flags & NETSOCK_IP ) {
    if( flags & NETSOCK_UDP ) {

      if( (sckt = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0 ) {
	LError("Unable to open socket");
        return -1;
      }
      
    } else if( flags & NETSOCK_TCP ) {

      if( (sckt = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 ) {
	LError("Unable to open socket");
        return -1;
      }
      
    } else {
      LError("Invalid IP socket type");
      errno = EINVAL;
      return -1;
    }
    
    if( fcntl( sckt, F_SETFL, O_NDELAY) < 0 ) {
      LError("set nodelay failed");
      close(sckt);
      return -1;
    }

    saddr.sin_family = AF_INET;
    saddr.sin_port = htons (port);
    inet_aton( addr, &saddr.sin_addr);

    if( flags & NETSOCK_SERVER ) {
      if( bind(sckt, (struct sockaddr *)&saddr, sizeof(saddr))) {
	LError("bind failed");
        close(sckt);
        return -1;
      }
      
      if( flags & NETSOCK_TCP ) {
        if( listen(sckt, 50)) {
	  LError("listen failed");
          close(sckt);
          return -1;
        }
      }
    } else if( flags & NETSOCK_CLIENT ) {
      if( connect(sckt, (struct sockaddr *)&saddr, sizeof(saddr))) {
	LError("conenct failed");
        close(sckt);
        return -1;
      }
    } else {
      LError("invalid socket flags");
      close(sckt);
      errno = EINVAL;
      return -1;
    }
  } else {
    LError("invalid socket type");
    errno = EINVAL;
    return -1;
  }

  return sckt;
}

void netsocket_destroy(s32_t fd) {
  close(fd);
}

