/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)association.c 1.5      04/02/21 Chris G. Davis"

/** \file src/structures/association.c
 *  \brief Association structure code
 *  \author Chris G. Davis 
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lous/association.h"
#include "internal/association.h"

association_t* association_alloc(s32_t keylen, s32_t datalen) {
  association_t* retval = NULL;

  errno = 0;

  Lmalloc( retval, sizeof ( association_t ), NULL);

  if( (retval->keytodata = hash_alloc(64, keylen, 
				      HASH_MULTIPLEX | HASH_SPLAY,
				      NULL, NULL, NULL)) == NULL ) {
    LError("unable to malloc hash");
    free( retval );
    return NULL;
  }

  if( (retval->datatokey = hash_alloc(64, datalen,
				      HASH_MULTIPLEX | HASH_SPLAY,
				      NULL, NULL, NULL)) == NULL ) {
    LError("unable to malloc hash");
    hash_destroy( retval->keytodata );
    free( retval );
    return NULL;
  }

  return retval;
}

void association_destroy(association_t* set) {

  hash_destroy( set->datatokey );
  hash_destroy( set->keytodata );

  free( set );

  errno = 0;

}

void association_create(association_t* set, void* key, void* data) {

  hash_insert( set->keytodata, key, data );
  hash_insert( set->datatokey, data, key );

  errno = 0;

}

void association_delete(association_t* set, void* key, void* data) {
  void* info;

  if( key ) {
    info = hash_remove( set->keytodata, key );
    hash_remove( set->datatokey, info );
  } else {
    info = hash_remove( set->datatokey, data );
    hash_remove( set->keytodata, info );
  }

  errno = 0;

}

void* association_getkey(association_t* set, void* data) {
  void* retval = NULL;

  errno = 0;

  retval = hash_lookup( set->datatokey, data );

  return retval;
}

void* association_getdata(association_t* set, void* data) {
  void* retval = NULL;

  errno = 0;

  retval = hash_lookup( set->keytodata, data);

  return retval;
}

