/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_STRUCTURES_INTERNAL_SPLAY_H
#define LOUS_SRC_STRUCTURES_INTERNAL_SPLAY_H

#ident "@(#)splay.h 1.5      04/02/16 Chris G. Davis"

/** \file src/structures/internal/splay.h
 *  \brief Splay Tree internal definitions 
 *  \author Chris G. Davis
 *
 */

/** \brief The Splay tree node structure
 *
 *  This structure contains one node of the splay tree.
 */
struct LOUS_Splay_Node {
  splay_node_t* left; /**< left child node */
  splay_node_t* right; /**< right child node */

  splay_node_t* forwards; /**< forwards link */
  splay_node_t* backwards; /**< backwards link */

  void* key; /**< node key, a string or keylen bytes */
  void* data; /**< node data */
};

/** \brief The splay tree structure
 * 
 * This structure contains the nodes that make up the splay tree
 */
struct LOUS_Splay_Tree {
  iterator_list_t iterate;
  comp_t (*comp_func)(void* keya, void* keyb, s32_t keylen);
  s32_t keylen; /**< length of node keys, or 0 if keys are strings */

  s32_t num_entries; /**< entries in tree */
  s32_t flags; 

  /**
   * compare \a keya and \a keyb and returns the comparision
   */

  void (*free_func)(void* data); /**< function for freeing node data */

  splay_node_t* nodes; /**< tree root */
};

splay_node_t* splay_lookup_ent(splay_tree_t* tree, void* key);

#endif /* */
