/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)basic.c 1.4      04/02/16 Chris G. Davis"

/** \file tests_t0003_basic_c
 *  \brief
 *  \author Chris G. Davis 
 *
*/

#include <stdio.h>

#include "lous/lous.h"
#include "lous/logging.h"
#include "lous/hash.h"

int main() {
	hash_t* hash;
	char* data;
	void** keys;
	s32_t index = 0;

        LOUS_Debug_Init(LOG_STDERR | LOG_NODATE | LOG_NOTIME,0);
        LOUS_SetDebug(LD_func, LL_veryverbose);
        LOUS_SetDebug(LD_debug, LL_veryverbose);
	hash = hash_alloc(32, 0, 0, NULL, NULL, NULL);

	printf("Size: %d\n", hash_get_buckets(hash));

	hash_destroy( hash );

	hash = hash_alloc(2356, 0, 0, NULL, NULL, NULL);

	printf("Size: %d\n", hash_get_buckets(hash));

	hash_destroy( hash );

	hash = hash_alloc(31, 0, 0, NULL, NULL, NULL);

	printf("Size: %d\n", hash_get_buckets(hash));

	hash_insert(hash, "key1", "data1");
	hash_insert(hash, "key2", "data2");
	hash_insert(hash, "key3", "data3");
	hash_insert(hash, "key4", "data4");
	hash_insert(hash, "key5", "data5");

	data = hash_lookup(hash, "key3");
	printf("Got: %s\n", data);

	data = hash_remove(hash, "key4");
	printf("Got: %s\n", data);

	data = hash_replace(hash, "key1", "datanew1");
	printf("Got: %s\n", data);

	data = hash_lookup(hash, "key1");
	printf("Got: %s\n", data);

	data = hash_remove(hash, "key2");
	printf("Got: %s\n", data);

	
	keys = (char**)hash_getkeys(hash);
	data = (char*)keys[index++];
	while( data != NULL ) {
		printf("Key: %s\n", data);
		data = (char*)keys[index++];
	}

	hash_destroy( hash );

	return 0;
}

