/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)basic.c 1.2      04/02/22 Chris G. Davis"

/** \file tests/t0009/basic.c
 *  \brief
 *  \author Chris G. Davis 
 *
*/
#include <stdio.h>
#include "lous/tokenizer.h"

int main() {
	tokenizer_t* toker;
	tokenizer_response_t* resp;
	s32_t ind;
	s32_t maxc;

        LOUS_Debug_Init(LOG_STDERR | LOG_NODATE | LOG_NOTIME,0);
        LOUS_SetDebug(LD_func, LL_veryverbose);
        LOUS_SetDebug(LD_debug, LL_veryverbose);

	toker = tokenizer_alloc();
	tokenizer_add_separator(toker, ' ');
	tokenizer_add_separator(toker, '\t');

	tokenizer_add_terminator(toker, '\n');
	tokenizer_add_ignored(toker, '\r');
	tokenizer_add_ignored(toker, ',');

	resp = tokenizer_tokenize(toker, "this is a test string, it has a terminator\n");

	/* now, go through the response */	
	maxc = tokenizer_get_numtokens( resp );
	printf("Got %d tokens\n", maxc);
	printf(" term: %d\n", tokenizer_get_terminator(resp));
	for(ind = 0; ind < maxc; ind ++ ) {
		printf(" Token: %d\n", ind);
		printf("   string: %s\n", tokenizer_get_string(resp, ind));
		printf("   preceeding: %s\n", tokenizer_get_preceeding(resp, ind));
		printf("   token: %s\n", tokenizer_get_token(resp, ind));
		printf("   remainder: %s\n", tokenizer_get_remainder(resp, ind));

	}

	tokenizer_destroy_response( resp );

	resp = tokenizer_tokenize(toker, "this is a test string, it has no terminator");

	/* now, go through the response */	
	maxc = tokenizer_get_numtokens( resp );
	printf("Got %d tokens\n", maxc);
	printf(" term: %d\n", tokenizer_get_terminator(resp));
	for(ind = 0; ind < maxc; ind ++ ) {
		printf(" Token: %d\n", ind);
		printf("   string: %s\n", tokenizer_get_string(resp, ind));
		printf("   preceeding: %s\n", tokenizer_get_preceeding(resp, ind));
		printf("   token: %s\n", tokenizer_get_token(resp, ind));
		printf("   remainder: %s\n", tokenizer_get_remainder(resp, ind));

	}

	tokenizer_destroy_response( resp );

	resp = tokenizer_tokenize(toker, "this is a test string, it     has \na terminator\n");

	/* now, go through the response */	
	maxc = tokenizer_get_numtokens( resp );
	printf("Got %d tokens\n", maxc);
	printf(" term: %d\n", tokenizer_get_terminator(resp));
	for(ind = 0; ind < maxc; ind ++ ) {
		printf(" Token: %d\n", ind);
		printf("   string: %s\n", tokenizer_get_string(resp, ind));
		printf("   preceeding: %s\n", tokenizer_get_preceeding(resp, ind));
		printf("   token: %s\n", tokenizer_get_token(resp, ind));
		printf("   remainder: %s\n", tokenizer_get_remainder(resp, ind));

	}

	tokenizer_destroy_response( resp );
	
	return 0;
}

