/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)command.c 1.2      04/02/22 Chris G. Davis"

/** \file tests/t0009/basic.c
 *  \brief
 *  \author Chris G. Davis 
 *
*/
#include <stdio.h>
#include "lous/cli-macro.h"

CLI_FUNC(first, test) {
  cli_printf("my first test");
}

CLI_FUNC(second, test) {
  cli_print("my second");
  cli_printf(" test");
}

CLI_FUNC(third, test) {
  cli_printf("my third test");
}

CLI_FUNC_NI(show, test);

CLI_FUNC(general, test_show) {
  cli_printf("my show general");
}

CLI_FUNC(ip, test_show) {
  cli_printf("my show ip");
}

CLI_FUNC(address, test_show_ip) {
  cli_printf("my show about address %s", cli_get_param(3));
}

CLI_FUNC(detail, test_show_ip_address) {
  cli_printf("my show detail about %s", cli_get_param(3));
}

CLI_FUNC(id, test_show) {
  cli_printf("my show id");
}

CLI_FUNC(ident, test_show_id) {
  cli_printf("my show id ident");
}

CLI_FUNC(again, test_show) {
  cli_printf("my show again");
}

CLI_FUNC(last, test) {
  cli_printf("my last");
}

CLI_EXPANDER(id_expand) {
}

cli_item_t* cli_init() {
  CLI_NEWROOT(test) {
    CLI_FUNCTION(first, test, "the first function", 0);
    CLI_FUNCTION(second, test, "the second function",0 );
    CLI_FUNCTION(third, test, "the third function", 0);
    CLI_BRANCH(show, test, "show info", 0) {
      CLI_FUNCTION(general, test_show, "show general info", 0);
      CLI_BRANCH(ip, test_show, "show info about ip address", 0) {
	CLI_PARAMBRANCH(address, test_show_ip, "^[0-9.]*$", NULL, "ip address", 0) {
	  CLI_FUNCTION(detail, test_show_ip_address, "show detail info", 0);
	}
      }
      CLI_BRANCH(id, test_show, "show info about id address", 0) {
	CLI_PARAM(ident, test_show_id, "33[[:xdigit:]]{14}", id_expand, "show info about id", 0);
      }
      CLI_FUNCTION(again, test_show, "again, test the sort", 0);
    }
    CLI_FUNCTION(last, test, "the last function", 0);
  }

  return test;
}

int main() {
  cli_item_t* test;
  cli_user_t* user;
  tokenizer_t* toker;
  char* resp;

  LOUS_Debug_Init(LOG_STDERR | LOG_NODATE | LOG_NOTIME,0);
  LOUS_SetDebug(LD_func, LL_veryverbose);
  LOUS_SetDebug(LD_debug, LL_veryverbose);

  toker = tokenizer_alloc();
  tokenizer_add_separator(toker, ' ');
  tokenizer_add_separator(toker, '\t');
  
  tokenizer_add_terminator(toker, '\n');
  tokenizer_add_ignored(toker, '\r');
  tokenizer_add_ignored(toker, ',');
  
  //resp = tokenizer_tokenize(toker, "this is a test string, it has a terminator\n");
  
  test = cli_init();

  user = cli_user_alloc( test, 255, 72);

  if( cli_user_run(user, toker, "first\n") == 0 ) {
    
    while( ( resp = cli_user_response_getline(user)) != NULL ) {
      printf("%s\n", resp);
    }
  }

  if( cli_user_run(user, toker, "second\n") == 0 ) {
    
    while( ( resp = cli_user_response_getline(user)) != NULL ) {
      printf("%s\n", resp);
    }
  }

  if( cli_user_run(user, toker, "show general\n") == 0 ) {
    
    while( ( resp = cli_user_response_getline(user)) != NULL ) {
      printf("%s\n", resp);
    }
  }

  if( cli_user_run(user, toker, "   show    general   \n") == 0 ) {
    
    while( ( resp = cli_user_response_getline(user)) != NULL ) {
      printf("%s\n", resp);
    }
  }

  if( cli_user_run(user, toker, "show ip 10.110.10.10\n") == 0 ) {
    
    while( ( resp = cli_user_response_getline(user)) != NULL ) {
      printf("%s\n", resp);
    }
  }

  if( cli_user_run(user, toker, "show ip a10.110.10.10\n") == 0 ) {
    
    while( ( resp = cli_user_response_getline(user)) != NULL ) {
      printf("%s\n", resp);
    }
  }

  if( cli_user_run(user, toker, "show ip 10.11.12.13 detail\n") == 0 ) {
    
    while( ( resp = cli_user_response_getline(user)) != NULL ) {
      printf("%s\n", resp);
    }
  }

  if( cli_user_run(user, toker, "not exist\n") == 0 ) {
    
    while( ( resp = cli_user_response_getline(user)) != NULL ) {
      printf("%s\n", resp);
    }
  }

  return 0;
}

